# Copyright 2018 eGym GmbH
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Wrapper for the Freshservice API.

Classes:
    API - Wrapper for basic FreshserviceAPI access via HTTP messages
    AgentAPI - Wrapper for Helpdesk administrators
    AssetAPI - Wrapper for CMDB items
    TaskAPI - Wrapper for Helpdesk tasks
    TicketAPI - Wrapper for Helpdesk tickets
    UserAPI - Wrapper for Helpdesk customers

"""
import logging

from requests import Request, Session

from .models import Asset, Ticket, Task, User, Agent
from .errors import (ResponseError,
                     AuthenticationFailed,
                     UserSearchFailed,
                     AgentSearchFailed)


class AssetAPI(object):

    """API for handling CMDB assets.

    An important note about the asset_type data structure: The
    custom_fields attribute wont be initialized right away. Doing so
    would require one API call for each asset type, which is a waste of
    ressources in general (because you can't assume that every asset
    type will actually be used). Instead the field will be initialized
    with None and will be loaded on request.
    """

    def __init__(self, api):
        self._api = api
        self._prefix = 'cmdb/'

        self._asset_types = self.get_asset_types()
        self.asset_types = {}

        for asset_type in self._asset_types:
            self.asset_types[asset_type['label']] = {
                'id': asset_type['id'],
                'custom_fields': None
            }

    def get_asset_types(self):
        """Returns a list of all asset types."""
        return self._api.get(self._prefix + 'ci_types.json')

    def get_custom_fields(self, type_id):
        """Return all domain specific fields for given asset type.

        The returned list consists of dictionaries with the name and ID
        of every custom field.
        """
        url = self._prefix + 'ci_types/{}.json'.format(type_id)
        custom_fields_raw = self._api.get(url)
        custom_fields = {}

        for field in custom_fields_raw:
            short_name = field['name'].strip('_{}'.format(field['ci_type_id']))
            custom_fields[short_name] = field['ci_type_id']

        return custom_fields

    def _load_custom_fields(self, asset_name):
        """Returns list of custom fields for given asset dictionary.

        First checks if the custom_fields are cached. If not, download
        the information and save it in a local data structure to prevent
        future API calls. Then return the list.
        """
        asset_type = self.asset_types[asset_name]

        if asset_type['custom_fields'] is None:
            custom_fields = self.get_custom_fields(asset_type['id'])
            self.asset_types[asset_name]['custom_fields'] = custom_fields

        return self.asset_types[asset_name]['custom_fields']

    def get(self, asset_id):
        """Return Asset object with given ID."""
        url = '{}items/{}.json'.format(self._prefix, asset_id)

        asset = self._api.get(url)
        asset_name = asset['config_item']['ci_type_name']
        custom_fields = self._load_custom_fields(asset_name)

        return Asset(custom_fields, **asset)

    def get_all(self, page_limit=200):
        """Return a list of all assets in the CMDB.

        The download will take a while and requires a lot of API calls,
        so please use the search function whenever possible. If you
        really need all assets, please save the result for later use.
        One page contains 50 asset objects.

        Keyword args:
            page_limit: Limits the number of pages, which will be downloaded.
                        This was added for safety reasons.
        """
        url = self._prefix + 'items.json?page={}'
        assets_raw = []
        assets = []
        page_number = 1

        while True:
            if page_number >= page_limit:
                logging.info('Reached page limit. Stopping download')
                logging.info('Downloaded %d pages of Asset information',
                             page_number - 1)
                break

            logging.debug('Downloading page %d', page_number)
            page = self._api.get(url.format(page_number))
            if not page:
                logging.debug('Page %d was empty.', page_number)
                logging.info('Downloaded %d pages of Asset information',
                             page_number - 1)
                break
            assets_raw += page
            page_number += 1

        for asset in assets_raw:
            custom_fields = self._load_custom_fields(asset['ci_type_name'])
            assets.append(Asset(custom_fields, **asset))

        return assets

    def search(self, field, value):
        """Returns a list of assets which match the search parameters.

        Args:
            field: The attribute that identifies the assets which are searched
                   for. Possible values are: 'name', 'asset_tag' and
                   'serial_number'. Note that a search for the 'name'
                   attribute will return all items with a name, that includes
                   the given value, not just those which match the name
                   exactly.
            value: The value that the field should have.
        """
        url = self._prefix + 'items/list.json?field={}&q={}'.format(field,
                                                                    value)
        assets_raw = self._api.get(url)
        assets = []
        for asset in assets_raw['config_items']:
            custom_fields = self._load_custom_fields(asset['ci_type_name'])
            assets.append(Asset(custom_fields, **asset))
        return assets

    def update(self, asset_id, **kwargs):
        """Use this function with care, it might overwrite data."""
        # get asset config
        asset = self.get(asset_id)
        custom_fields = self._load_custom_fields(asset.ci_type_name)
        if 'name' not in kwargs.keys():
            kwargs['name'] = asset.name

        # rename custom fields
        delete_after = []
        kwargs['level_field_attributes'] = {}
        for key, value in kwargs.items():
            if key in custom_fields.keys():
                new_name = '{}_{}'.format(key, custom_fields[key])
                kwargs['level_field_attributes'][new_name] = value
                delete_after.append(key)
        for key in delete_after:
            del kwargs[key]

        url = self._prefix + '/items/{}.json'.format(asset_id)
        asset = self._api.put(url, data={'cmdb_config_item': kwargs})
        return Asset(custom_fields, **asset)


class TicketAPI(object):

    """API for handling Tickets."""

    def __init__(self, api):
        self._api = api
        self._tickets_prefix = 'helpdesk/tickets'

        self.api_postfix = ''
        self.custom_fields = []

        ticket_fields = self.get_ticket_fields()
        for field in ticket_fields:
            field = field['ticket_field']
            if field['default'] is True:
                continue
            name = field['name'].split('_')
            self.api_postfix = '_' + name[len(name) - 1]
            short_name = field['name'].replace(self.api_postfix, '')
            self.custom_fields.append(short_name)

    def get_ticket_fields(self):
        """Used for getting domain specific ticket fields."""
        return self._api.get('ticket_fields.json')

    def get_ticket(self, ticket_id):
        """Return ticket object with given id."""
        url = self._tickets_prefix + '/{}.json'.format(ticket_id)
        ticket = self._api.get(url)
        return Ticket(self.custom_fields, self.api_postfix, **ticket)

    def create_ticket(self, subject, email, **kwargs):
        """Create a ticket.

        due_by must be passed in ISO 8601 format and may not
        be older than the current datetime.

        Args:
            subject: The subject and title of the ticket.
            email: The email address of the requester.
            kwargs: Any other fields can be passed as <key>=<value>
                pairs. Please refer to the Freshservice API
                documentation to see, which fields and values are legit.
                You can also compare with the content of any ticket.json
                file. Note, that the fields "status", "priority" and
                "source" use numeric values.
                They are defined in models.py.

        """
        url = self._tickets_prefix + '.json'

        data = self.create_freshservice_dict(kwargs)
        data['helpdesk_ticket']['email'] = email
        data['helpdesk_ticket']['subject'] = subject

        ticket = self._api.post(url, data)
        return Ticket(self.custom_fields, self.api_postfix, **ticket)

    def update_ticket(self, ticket_id, **kwargs):
        """Update a ticket.

        This function does not return a ticket object, because
        the JSON, which is returned by the API is not compliant
        with standard JSON format for Tickets.

        This method works the same as create_ticket, except that you are
        overwriting fields of an already existing ticket.

        Please note that it is not possible to update the description of a
        ticket.

        There is an update specific quirk, that you need to set
        manual_due_by to true, before you can update the due date.

        """
        url = self._tickets_prefix + '/{}.json'.format(ticket_id)
        data = self.create_freshservice_dict(kwargs)
        self._api.put(url, data)

    def create_freshservice_dict(self, kwargs):
        """Create data object for creating and updating tickets.

        The Freshservice API has some weird quirks, some of which are
        not even documented and can only be found in forums. This method
        is handling these special cases.
        """
        data = {'helpdesk_ticket': {}}

        if 'tags' in kwargs:
            data['helpdesk'] = {}
            data['helpdesk']['tags'] = ','.join(kwargs['tags'])
            del kwargs['tags']

        if 'cc_email' in kwargs:
            data['cc_emails'] = ','.join(kwargs['cc_email'])
            del kwargs['cc_email']

        if 'reply_cc' in kwargs:
            data['reply_cc'] = kwargs['reply_cc']
            del kwargs['reply_cc']

        data['helpdesk_ticket']['manual_dueby'] = True
        if 'due_by' in kwargs:
            data['helpdesk_ticket']['frDueBy'] = kwargs['due_by']

        data, kwargs = self.rename_custom_fields(data, kwargs)
        data['helpdesk_ticket'].update(kwargs)
        return data

    def rename_custom_fields(self, data, kwargs):
        """Automatically pack custom fields to make the interface simpler."""
        custom_fields = {}
        delete_after = []
        for field in kwargs.keys():
            if field in self.custom_fields:
                custom_fields[field + self.api_postfix] = kwargs[field]
                delete_after.append(field)
        for field in delete_after:
            del kwargs[field]

        data['helpdesk_ticket']['custom_field'] = custom_fields
        return data, kwargs

    def delete_ticket(self, ticket_id):
        """Delete a ticket."""
        url = self._tickets_prefix + '/{}.json'.format(ticket_id)
        self._api.delete(url)

    def get_ticket_list_by_view(self, view_id):
        """Get a list of tickets, predefined by a view.

        Args:
            view_id: To find the view_id, select a view in Helpdesk and
                     you will see the ID in the last part of the URL.
        """
        url = self._tickets_prefix + '/view/{}?format=json'.format(view_id)
        raw_list = self._api.get(url)
        ticket_list = []
        for ticket in raw_list:
            ticket_list.append(
                Ticket(self.custom_fields, self.api_postfix, **ticket))
        return ticket_list

    def add_note(self, ticket_id, description, public=False):
        """Add a note to the ticket conversation."""
        url = self._tickets_prefix + \
            '/{}/conversations/note.json'.format(ticket_id)
        data = {'helpdesk_note': {'body_html': description}}
        if public:
            data['helpdesk_note']['private'] = False
        self._api.post(url, data)


class TaskAPI(object):

    """Wrapper for Helpdesk tasks.

    Freshservice uniquely identifies a Task with the fields "ticket_id"
    and "id" in JSON. Tasks can be used for various modules in
    Freshservice, Helpdesk tickets are the most common use case.
    """

    def __init__(self, api, module='tickets'):
        """Constructor of TaskAPI.

        Args:
            api: An instance of API
            module(optional): May be one of: 'tickets', 'problems',
                              'changes', 'releases'.

        """
        self._api = api
        self._task_prefix = 'itil/{}/'.format(module)

    def create_task(self, title, description, item_id, **kwargs):
        """Create a task.

        Args:
            title: The title of the task
            description: The description in plain text, HTML is not supported.
            item_id: The "owner" of the ticket, for example the ID of a ticket.
        """
        url = self._task_prefix + '{}/it_tasks.json'.format(item_id)
        data = {'title': title, 'description': description}
        data.update(kwargs)
        task = self._api.post(url, {'it_task': data})
        return Task(**task)

    def update_task(self, item_id, task_id, **kwargs):
        """Update a task."""
        url = self._task_prefix + \
            '{}/it_tasks/{}.json'.format(item_id, task_id)
        self._api.put(url, data={'it_task': kwargs})

    def get_task(self, item_id, task_id):
        """Get a task."""
        url = self._task_prefix + \
            '{}/it_tasks/{}.json'.format(item_id, task_id)
        task = self._api.get(url)
        return Task(**task)

    def get_all_tasks(self, item_id):
        """Get all tasks for a given item ID.

        Returns a list of Tasks.
        """
        url = self._task_prefix + '{}/it_tasks.json'.format(item_id)
        task_list = self._api.get(url)
        tasks = []
        for task in task_list:
            tasks.append(Task(**task))
        return tasks

    def delete_task(self, item_id, task_id):
        """Delete a task."""
        url = self._task_prefix + \
            '{}/it_tasks/{}.json'.format(item_id, task_id)
        self._api.delete(url)


class UserAPI(object):

    """Very basic access to user information"""

    def __init__(self, api):
        self._api = api

    def search(self, email):
        """Returns user with given email

        Note that agents are not considered to be users and cannot be
        searched with this API.

        Raises:
            UserSearchFailed: Will be raised, if the user doesn't exist, or
                              the API returns multiple objects for the same
                              email.
        """
        url = 'itil/requesters.json?query=email%20is%20{}'.format(email)
        user = self._api.get(url)
        if len(user) != 1:
            raise UserSearchFailed
        return User(**user[0])


class AgentAPI(object):

    """Very basic access to Freshservice admin information"""

    def __init__(self, api):
        self._api = api

    def search(self, email):
        """Returns agent with given email

        Raises:
            AgentSearchFailed: Will be raised, if the user doesn't exist, or
                               the API returns multiple objects for the same
                               email.
        """
        url = 'agents.json?query=email%20is%20{}'.format(email)
        agent = self._api.get(url)
        if len(agent) != 1:
            raise AgentSearchFailed
        return Agent(**agent[0])


class API(object):

    """Freshservice API wrapper.

    For the most part, this is a wrapper for HTTP calls to Freshservice.
    For each required HTTP function there is a method, which basically
    creates the URL string and then calls _action with the function name.

    It also saves the API key to make authentication possible. Cookies
    will always be cleared, because the API instantly resets every
    session.

    """

    def __init__(self, api_key, domain):
        """Creates an object of API

        Args:
            api_key: The API key of an agent with sufficient rights.
            domain: The name of your Freshservice instance.
                    Example: If the URL of your instance is
                    https://domain.freshservice.com, you should pass 'domain'.

        """

        self._api_prefix = 'https://{}.freshservice.com/'.format(domain)
        self._session = Session()
        self._session.auth = (api_key, '')
        self._session.headers = {
            'Content-Type': 'application/json',
            'Connection': 'close'
        }

        # debug variables
        self.response = None
        self.request = None

    def _action(self, req):
        """Prepare and send the request, then handle errors.

        This method is used all accross this module. It provides
        functionality that all HTTP methods need. After making an API
        call, it will analyze the JSON response, look for errors and
        throw an exception if necessary.

        Raises:
            AuthenticationFailed: The API object was initialized with an
                                  invalid API key, or the associated agent
                                  does not have sufficient rights.
            ResponseError : Response is not in JSON format, or contains the
                           'error'/'errors' field.
        """
        self._session.cookies.clear()
        self.request = self._session.prepare_request(req)

        resp = self._session.send(self.request)
        try:
            j = resp.json()
            self.response = j
        except Exception:
            self.response = None
            raise ResponseError(self.request, self.response)

        if 'error' in j or 'errors' in j:
            raise ResponseError(self.request, self.response)

        if 'require_login' in j:
            raise AuthenticationFailed

        try:
            resp.raise_for_status()
        except Exception:
            raise ResponseError(self.request, self.response)

        return j

    def get(self, url):
        """Wrapper around request.get().

        Returns a dictionary with the parsed response.
        """
        req = Request('GET', self._api_prefix + url)
        return self._action(req)

    def post(self, url, data=None):
        """Wrapper around request.post().

        Returns a dictionary with the parsed response.
        """
        if data is None:
            data = {}
        req = Request('POST', self._api_prefix + url, json=data)
        return self._action(req)

    def put(self, url, data=None):
        """Wrapper around request.put().

        Returns a dictionary with the parsed response.
        """
        if data is None:
            data = {}
        req = Request('PUT', self._api_prefix + url, json=data)
        return self._action(req)

    def delete(self, url):
        """Wrapper around request.delete().

        Returns a dictionary with the parsed response.
        """
        req = Request('DELETE', self._api_prefix + url)
        return self._action(req)



# Copyright 2018 eGym GmbH
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Defines errors thrown by lib-freshservice."""
import logging


class FreshserviceError(Exception):
    """An error occurred in the Freshservice module."""


class BadInput(FreshserviceError):
    """The user passed something, that doesn't match the model."""


class AuthenticationFailed(FreshserviceError):
    """Invalid or missing API key."""


class UnexpectedValue(FreshserviceError):
    """The API passed a value, that this wrapper doesn't understand."""


class UserSearchFailed(FreshserviceError):
    """Couldn't find user with given email"""


class AgentSearchFailed(FreshserviceError):
    """Couldn't find agent with given email"""


class ResponseError(FreshserviceError):

    """Response is not JSON or contains errors."""

    def __init__(self, request, response):
        """Log error and raise parent error

        Args:
            request: The request object.
            response: The response string in JSON format. Pass None or '' in
                case the response was not in JSON.
        """
        super().__init__()
        self.request = request
        self.response = response

        logging.error('-----------Request-----------\n' + str(request.body))
        if response:
            logging.error('-----------Response-----------\n' + str(response))
        else:
            logging.error('The Freshservice response is not in JSON format')

    def __str__(self):
        return self.request.method + ' ' + self.request.url



# Copyright 2018 eGym GmbH
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Basic datastructures for the use of the Freshservice API.

Classes:
    FreshserviceModel - Base class for common operations
    Agent - Class for Helpdesk admins
    Asset - Class for CMDB items
    Task - Convenient representation of a Helpdesk task
    Ticket - Convenient representation of a Helpdesk ticket
    User - Class for users

"""
from .errors import BadInput, UnexpectedValue


class FreshserviceModel(object):

    """Base class for the FreshserviceAPI data model.

    All other model classes inherit this. It provides basic
    functionality to initialize class attributes and common methods.

    """

    def __init__(self, inner_dictionary_names, **kwargs):
        """Constructor of FreshserviceModel.

        This method sets the class attributes with the content of
        kwargs. Some attributes will be saved private, to make them
        accessible more conveniently via properties.

        Args:
            inner_dictionary_names (list): A list of names, which
                represent dictionaries in kwargs. Every dictionary in
                this list will be popped, so that kwargs doesn't contain
                to many inner dictionaries. As a result the models
                __dict__ will have less inner levels.
            kwargs (dict): It is expected, that a dictionary is used,
                which is obtained by parsing JSON, passed by the
                Freshservice API.

        """
        for k in inner_dictionary_names:
            if k in kwargs.keys() and len(kwargs[k]) > 0:
                kwargs.update(kwargs.pop(k))

        for k, v in kwargs.items():
            if hasattr(self.__class__, k):
                k = '_' + k
            setattr(self, k, v)

    def __str__(self):
        """This method only prints the content of its dictionary."""
        return str(self.__dict__)

    def __repr__(self):
        """Create object representation.

        With this you can create a duplicate of the object with eval.
        Note, that this is probably not the same code, that actually
        created this object.

        """
        representation = self.__class__.__name__ + '('
        not_first = False
        for k, v in self.__dict__.items():
            if not_first:
                representation += ', '
            not_first = True
            representation += k + ' = ' + repr(v)
        representation += ')'
        return representation


class Asset(FreshserviceModel):

    def __init__(self, custom_fields, **kwargs):
        super().__init__(
            inner_dictionary_names=[
                'item', 'config_item', 'levelfield_values'
            ],
            **kwargs)

        for key, value in custom_fields.items():
            long_name = '{}_{}'.format(key, value)
            setattr(self, key, getattr(self, long_name))
            delattr(self, long_name)


class Ticket(FreshserviceModel):

    """The ticket model.

    This class represents the ticket data model of Freshservice. It
    provides wrapper classes for better usability of the priority,
    status and source classes. Due to inconsistencies in the
    Freshservice API, it is currently only used for HTTP GET.
    """

    # Ticket source
    EMAIL = 1
    PORTAL = 2
    PHONE = 3
    CHAT = 4

    # Ticket status
    OPEN = 2
    PENDING = 3
    RESOLVED = 4
    CLOSED = 5

    # Ticket Priority
    LOW = 1
    MEDIUM = 2
    HIGH = 3
    URGENT = 4

    def __init__(self, custom_fields=None, api_postfix='', **kwargs):
        """Constructor of Ticket.

        This method calls the constructor of the FreshserviceModel. It
        also renames some attributes for ease of use.
        """
        if custom_fields is None:
            custom_fields = []

        super().__init__(['item', 'helpdesk_ticket', 'custom_field'], **kwargs)

        for field in custom_fields:
            setattr(self, field, getattr(self, field + api_postfix))
            delattr(self, field + api_postfix)

    @property
    def priority(self):
        """Return string representation of priority.

        This returns either 'low', 'medium', 'high', or 'urgent'. The
        API itself saves numeric values. Alternatively you may use the
        field 'priority_name'.
        """
        _p = {
            self.LOW: 'low',
            self.MEDIUM: 'medium',
            self.HIGH: 'high',
            self.URGENT: 'urgent'
        }
        try:
            return _p[self._priority]
        except KeyError:
            raise UnexpectedValue('Priority: {}'.format(self._priority))

    @priority.setter
    def priority(self, value):
        """Set priority with string representation.

        The setter expects one of the following values: 'low', 'medium',
        'high', 'urgent'
        """
        _p = {
            'low': self.LOW,
            'medium': self.MEDIUM,
            'high': self.HIGH,
            'urgent': self.URGENT
        }
        try:
            self._priority = _p[value]
        except KeyError:
            raise BadInput('Priority: {}.'.format(value))

    @property
    def status(self):
        """Return string representation of status.

        This returns either 'open', 'pending', 'resolved', or 'closed'.
        The API itself saves numeric values. Alternatively you may use
        the field 'status_name'.
        """
        _s = {
            self.OPEN: 'open',
            self.PENDING: 'pending',
            self.RESOLVED: 'resolved',
            self.CLOSED: 'closed'
        }
        try:
            return _s[self._status]
        except KeyError:
            raise UnexpectedValue('Status: {}'.format(self._status))

    @status.setter
    def status(self, value):
        """Set status with string representation.

        The setter expects one of the following values: 'open',
        'pending', 'resolved', or 'closed'
        """
        _s = {
            'open': self.OPEN,
            'pending': self.PENDING,
            'resolved': self.RESOLVED,
            'closed': self.CLOSED
        }
        try:
            self._status = _s[value]
        except KeyError:
            raise BadInput('Status: {}'.format(value))

    @property
    def source(self):
        """Return string representation of source.

        This returns either 'email', 'portal', 'phone', or 'chat'. The
        API itself saves numeric values. Alternatively you may use the
        field 'source_name'.
        """
        _s = {
            self.EMAIL: 'email',
            self.PORTAL: 'portal',
            self.PHONE: 'phone',
            self.CHAT: 'chat'
        }
        try:
            return _s[self._source]
        except KeyError:
            raise UnexpectedValue('Source: {}'.format(self._source))

    @source.setter
    def source(self, value):
        """Set source with string representation.

        The setter expects one of the following values: 'email',
        'portal', 'phone', 'chat'
        """
        _s = {
            'email': self.EMAIL,
            'portal': self.PORTAL,
            'phone': self.PHONE,
            'chat': self.CHAT
        }
        try:
            self._source = _s[value]
        except KeyError:
            raise BadInput('Source: {}'.format(value))


class Task(FreshserviceModel):

    """The task model.

    This is very similar to the ticket model, just a little simplified.
    Therefore most of the documentation also applies here.
    """

    # Task status
    OPEN = 1
    INPROGRESS = 2
    COMPLETED = 3

    def __init__(self, **kwargs):
        """Constructor for creating a Task object."""
        super().__init__(['item', 'it_task'], **kwargs)

    @property
    def status(self):
        """Get a string representation of the status."""
        _s = {
            self.OPEN: 'open',
            self.INPROGRESS: 'inprogress',
            self.COMPLETED: 'completed'
        }
        try:
            return _s[self._status]
        except KeyError:
            raise UnexpectedValue('Status: {}'.format(self._status))

    @status.setter
    def status(self, value):
        """Set the status with a string representation."""
        _s = {
            'open': self.OPEN,
            'inprogress': self.INPROGRESS,
            'completed': self.COMPLETED
        }
        try:
            self._status = _s[value]
        except KeyError:
            raise BadInput('Status: {}'.format(value))


class User(FreshserviceModel):

    def __init__(self, **kwargs):
        super().__init__(['user'], **kwargs)


class Agent(FreshserviceModel):

    def __init__(self, **kwargs):
        super().__init__(['agent'], **kwargs)



# Copyright 2018 eGym GmbH
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os

from unittest import TestCase

from freshservice.api import API, TaskAPI, TicketAPI
from freshservice.errors import ResponseError
from freshservice.models import Task


TEST_DOMAIN = 'flamunda'
AGENT_ID = 7000027089


class TestTask(TestCase):

    @classmethod
    def setUpClass(cls):
        cls.api = API(os.environ['FRESHSERVICE_API_KEY'], TEST_DOMAIN)
        cls.ticket_api = TicketAPI(cls.api)
        cls.task_api = TaskAPI(cls.api)

        # create reference ticket
        cls.ticket_id = cls.ticket_api.create_ticket(
            'Test', 'requester@egym.de').display_id
        # create reference task
        cls.task = cls.task_api.create_task(
            'Test', 'description', cls.ticket_id, owner_id=AGENT_ID)

    def test_delete(self):
        task_id = self.task_api.create_task(
            'Test', 'description', self.ticket_id).id
        self.task_api.delete_task(self.ticket_id, task_id)
        with self.assertRaises(ResponseError):
            self.task_api.get_task(self.ticket_id, task_id)

    def test_update_and_get(self):
        self.assertEqual(self.task.description, 'description')
        self.assertEqual(self.task.owner_id, AGENT_ID)
        self.assertEqual(self.task.title, 'Test')
        self.assertEqual(self.task.status, 'open')

        self.task_api.update_task(
            self.ticket_id, self.task.id, status=Task.COMPLETED)

        self.task = self.task_api.get_task(
            self.ticket_id, self.task.id)
        self.assertEqual(self.task.status, 'completed')

    def test_magic_methods(self):
        print(self.task)
        print(repr(self.task))

    def test_get_all_tasks(self):
        tasks = self.task_api.get_all_tasks(self.ticket_id)
        self.assertEqual(len(tasks), 1)
        self.assertEqual(tasks[0].title, "Test")

    def test_error(self):
        with self.assertRaises(ResponseError):
            task_api = TaskAPI(self.api, module="none")
            task_api.create_task(" ", " ", 0)



# Copyright 2018 eGym GmbH
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from unittest import TestCase

from freshservice.api import API, TicketAPI
from freshservice.errors import BadInput
from freshservice.models import Ticket


TEST_DOMAIN = 'flamunda'


class TestTicket(TestCase):

    @classmethod
    def setUpClass(cls):
        cls.api = API(os.environ['FRESHSERVICE_API_KEY'], TEST_DOMAIN)
        cls.ticket_api = TicketAPI(cls.api)

        # create reference ticket
        ticket_data = {
            'description': 'Test',
            'status': Ticket.CLOSED,
            'priority': Ticket.LOW,
            'source': Ticket.PORTAL,
            'cc_email': 'test@egym.de',
            'reply_cc': 'test@egym.de'
        }
        cls.ticket = cls.ticket_api.create_ticket(
            'Test', 'requester@egym.de', **ticket_data)

    def test_add_note(self):
        self.ticket_api.add_note(
            self.ticket.display_id, description="Test", public=True)
        self.ticket_api.add_note(
            self.ticket.display_id, description="Test2")

    def test_delete(self):
        self.assertFalse(self.ticket.deleted)
        self.ticket_api.delete_ticket(self.ticket.display_id)
        self.ticket = self.ticket_api.get_ticket(self.ticket.display_id)
        self.assertTrue(self.ticket.deleted)

    def test_magic_methods(self):
        print(repr(self.ticket))
        print(str(self.ticket))

    def test_properties(self):
        ticket = eval(repr(self.ticket))
        assert hasattr(ticket, 'coordinate')
        assert hasattr(ticket, 'location')
        assert hasattr(ticket, 'type')
        ticket.priority = 'urgent'
        ticket.status = 'open'
        ticket.source = 'chat'
        self.assertEqual(ticket.priority, 'urgent')
        self.assertEqual(ticket.status, 'open')
        self.assertEqual(ticket.source, 'chat')
        with self.assertRaises(BadInput):
            ticket.priority = 'bla'
        with self.assertRaises(BadInput):
            ticket.status = 'blop'
        with self.assertRaises(BadInput):
            ticket.source = 'blurb'

    def test_update_and_get(self):
        self.assertIsInstance(self.ticket, Ticket)
        self.assertEqual(self.ticket.subject, 'Test')
        self.assertEqual(self.ticket.description, 'Test')
        self.assertEqual(self.ticket.status, 'closed')
        self.assertEqual(self.ticket.priority, 'low')
        self.assertEqual(self.ticket.source, 'portal')

        self.ticket_api.update_ticket(
            self.ticket.display_id,
            subject='Test update',
            due_by='2100-01-01')

        self.ticket = self.ticket_api.get_ticket(self.ticket.display_id)
        self.assertIsInstance(self.ticket, Ticket)
        self.assertEqual(self.ticket.subject, 'Test update')
        self.assertEqual(self.ticket.due_by, '2100-01-01T00:00:00+01:00')



# Copyright 2018 eGym GmbH
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from unittest import TestCase

from freshservice.api import API, AgentAPI, UserAPI
from freshservice.errors import AgentSearchFailed, UserSearchFailed


TEST_DOMAIN = 'flamunda'
AGENT_ID = 7000006217
USER_ID = 7000479601


class TestAgent(TestCase):

    @classmethod
    def setUpClass(cls):
        cls.api = API(os.environ['FRESHSERVICE_API_KEY'], TEST_DOMAIN)
        cls.agent_api = AgentAPI(cls.api)

    def test_agent_search(self):
        agent = self.agent_api.search('lennart.weiss@egym.de')
        self.assertEqual(agent.id, AGENT_ID)

    def test_agent_search_error(self):
        with self.assertRaises(AgentSearchFailed):
            self.agent_api.search('hello.world@egym.de')


class TestUser(TestCase):

    @classmethod
    def setUpClass(cls):
        cls.api = API(os.environ['FRESHSERVICE_API_KEY'], TEST_DOMAIN)
        cls.user_api = UserAPI(cls.api)

    def test_user_search(self):
        user = self.user_api.search('hello.world@egym.de')
        self.assertEqual(user.id, USER_ID)

    def test_user_search_error(self):
        with self.assertRaises(UserSearchFailed):
            self.user_api.search('nonexisting.user@egym.de')



# Copyright 2018 eGym GmbH
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from unittest import TestCase

from freshservice.api import API, AssetAPI


TEST_DOMAIN = 'flamunda'


class TestAsset(TestCase):

    @classmethod
    def setUpClass(cls):
        cls.api = API(os.environ['FRESHSERVICE_API_KEY'], TEST_DOMAIN)
        cls.asset_api = AssetAPI(cls.api)

    def test_asset_get(self):
        asset = self.asset_api.get(1)
        self.assertEqual(asset.description, 'Test')
        self.assertEqual(asset.user_id, None)
        self.assertEqual(asset.warranty, 0)
        self.assertEqual(asset.os, 'Mac')
        self.assertEqual(asset.screen_width, '13')
        self.assertEqual(asset.hostname, 'pc-657')

    def test_asset_search(self):
        assets = self.asset_api.search('name', 'pc-657')
        self.assertEqual(assets[0].hostname, 'pc-657')
        assets = self.asset_api.search('serial_number', 'SN_XYZ')
        self.assertEqual(assets[0].serial_number, 'SN_XYZ')

    def test_asset_get_all(self):
        assets = self.asset_api.get_all()
        print('We have %d assets in our inventory!' % len(assets))
        for asset in assets:
            if hasattr(asset, 'hostname'):
                if asset.hostname == 'pc-657':
                    self.assertEqual(asset.serial_number, 'SN_XYZ')
                    break

    def test_asset_update(self):
        asset = self.asset_api.update(1, screen_width='15', impact='1',
                                      mac_address='AA:AA:AA:AA:AA:AA')
        self.assertEqual(asset.impact, 1)
        self.assertEqual(asset.screen_width, '15')
        self.assertEqual(asset.mac_address, 'AA:AA:AA:AA:AA:AA')

        asset = self.asset_api.update(1, screen_width='13', impact='2',
                                      mac_address='')
        self.assertEqual(asset.impact, 2)
        self.assertEqual(asset.screen_width, '13')
        self.assertEqual(asset.mac_address, None)



# Copyright 2018 eGym GmbH
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="freshservice-wrapper",
    version="1.4",
    author="Lennart Weiss",
    author_email="lennart.weiss@egym.de",
    description="An API wrapper for Freshservice",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://bitbucket.org/egym-com/freshservice-wrapper/",
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ),
    install_requires=['requests'],
    python_requires='>=3'
)
